/*************************************************************************************
			NOTEMIDITEST - a test utility for NOTEMIDI

		(C) 2001 by Michael Minn - michael@michaelminn.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

*************************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>

#define SERIAL_PORT "/dev/ttyS0"
#define RAW_MIDI_DEVICE "/dev/midi03"
#define SEQUENCER_DEVICE "/dev/sequencer2"

int main(int argc, char **argv)
{
	printf("NOTEMIDITEST - a test utility for NOTEMIDI\nv2001.08.17 by Michael Minn\n");

	struct termios port_attributes;
	port_attributes.c_lflag = 0;
	port_attributes.c_lflag = NOFLSH;
	port_attributes.c_iflag = IGNBRK | IGNPAR;
	port_attributes.c_oflag = 0;
	port_attributes.c_cflag =  B38400 | CREAD | CSIZE | CS8 | CLOCAL | HUPCL;
	port_attributes.c_cc[VEOL] = '\r';
	port_attributes.c_cc[VERASE] = 0;
	port_attributes.c_cc[VKILL] = 0;
	port_attributes.c_cc[VMIN] = 0;
	port_attributes.c_cc[VTIME] = 0;

	unsigned char raw_midi_data[30] = { 0x80, 0x47, 0x40, 0x90, 0x40, 0x40,
										0x80, 0x40, 0x40, 0x90, 0x42, 0x40,
										0x80, 0x42, 0x40, 0x90, 0x44, 0x40,
										0x80, 0x44, 0x40, 0x90, 0x45, 0x40,
										0x80, 0x45, 0x40, 0x90, 0x47, 0x40 };

	printf("Testing " SERIAL_PORT "\n");
	int device = open(SERIAL_PORT, O_WRONLY | O_NOCTTY);
	if (device < 0)
	{
		perror("Failure opening " SERIAL_PORT);
		printf("You must be superuser to test the serial port\n");
	}

	else if (tcgetattr(device, &port_attributes) < 0)
		perror("Failure in tcgetattr getting serial port attributes for " SERIAL_PORT);

	else if (cfsetispeed(&port_attributes, B38400) < 0)
		perror("Failure in cfsteispeed setting serial port speed for " SERIAL_PORT);

	else if (cfsetospeed(&port_attributes, B38400) < 0)
		perror("Failure in cfsteospeed setting serial port speed for " SERIAL_PORT);

	else if (tcsetattr(device, TCSAFLUSH, &port_attributes) < 0)
		perror("Failure in tcsetattr setting serial port attributes for " SERIAL_PORT);

	else for (int raw_midi = 0; raw_midi < 30; raw_midi += 6)
		if (write(device, &raw_midi_data[raw_midi], 6) < 6)
		{
			perror("Failure writing to " SERIAL_PORT);
			return 0;
		}
		else
			usleep(500000);



	printf("Testing " RAW_MIDI_DEVICE "\n");

	if ((device = open(RAW_MIDI_DEVICE, O_WRONLY)) < 0)
	{
		perror("Failure opening " RAW_MIDI_DEVICE);
		printf("Make sure NOTEMIDI is listed in the following module listing\n");
		printf("or see README.html for mknod instructions\n");
		execlp("cat", "cat", "/proc/modules", NULL);
		return 0;
	}

	printf("Playing scale to " RAW_MIDI_DEVICE "\n");

	for (int raw_midi = 0; raw_midi < 30; raw_midi += 6)
		if (write(device, &raw_midi_data[raw_midi], 6) < 6)
		{
			perror("Failure writing to " RAW_MIDI_DEVICE);
			return 0;
		}
		else
			usleep(500000);

	close(device);

	if ((device = open(SEQUENCER_DEVICE, O_RDWR)) < 0)
	if (device < 0)
	{
		perror("Failure opening " SEQUENCER_DEVICE);
		printf("see README.html for mknod instructions\n");
		return 0;
	}

	int time_base = 256;
	if (ioctl(device, SNDCTL_TMR_TIMEBASE, &time_base) < 0)
	{
		perror("Failure setting sequencer ticks per beat\n");
		return 0;
	}

	int tempo = 120;
	if (ioctl(device, SNDCTL_TMR_TEMPO, &tempo) < 0)
	{
		perror("Failure setting sequencer tempo\n");
		return 0;
	}

	unsigned char sequencer_data[144] = { EV_TIMING, TMR_START, 0, 0, 0, 0, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEOFF, 0, 0x47, 0x40, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEON, 0, 0x40, 0x40, 0, 0,
										EV_TIMING, TMR_WAIT_ABS, 0, 0, 0, 1, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEOFF, 0, 0x40, 0x40, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEON, 0, 0x42, 0x40, 0, 0,
										EV_TIMING, TMR_WAIT_ABS, 0, 0, 0, 2, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEOFF, 0, 0x42, 0x40, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEON, 0, 0x44, 0x40, 0, 0,
										EV_TIMING, TMR_WAIT_ABS, 0, 0, 0, 3, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEOFF, 0, 0x44, 0x40, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEON, 0, 0x45, 0x40, 0, 0,
										EV_TIMING, TMR_WAIT_ABS, 0, 0, 0, 4, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEOFF, 0, 0x45, 0x40, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEON, 0, 0x47, 0x40, 0, 0,
										EV_TIMING, TMR_WAIT_ABS, 0, 0, 0, 5, 0, 0,
										EV_CHN_VOICE, 0, MIDI_NOTEOFF, 0, 0x47, 0x40, 0, 0,
										EV_TIMING, TMR_STOP, 0, 0, 0, 0, 0, 0 };

	printf("Playing scale on " SEQUENCER_DEVICE "\n");
	if (write(device, &sequencer_data, sizeof(sequencer_data)) < sizeof(sequencer_data))
	{
		perror("Failure writing to " SEQUENCER_DEVICE);
		return 0;
	}
	sleep(3);

	printf("NOTEMIDI successfully tested\n");

	return 0;
}
